/**
 * Copyright (c) 2020, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * WSO2 Inc. licenses this file to you under the Apache License,
 * Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
import classNames from "classnames";
import React from "react";
import { Label } from "semantic-ui-react";
import { Heading } from "../typography";
/**
 * Product Brand component.
 *
 * @param {React.PropsWithChildren<ProductBrandPropsInterface>} props - Props injected to the component.
 *
 * @return {React.ReactElement}
 */
export const ProductBrand = (props) => {
    const { children, className, logo, name, style, version, ["data-testid"]: testId } = props;
    const versionLabelClasses = classNames("version-label", {
        "primary": !version.labelColor,
        [version.labelColor]: version.labelColor === "primary" || version.labelColor === "secondary"
    });
    /**
     * Resolves the version label color.
     *
     * @return {SemanticCOLORS} Resolved color.
     */
    const resolveVersionLabelColor = () => {
        if (version.labelColor
            && !(version.labelColor === "auto"
                || version.labelColor === "primary"
                || version.labelColor === "secondary")) {
            return version.labelColor;
        }
        if (version.labelColor === "primary" || version.labelColor === "secondary") {
            return "orange";
        }
        if (version.releaseType === "alpha") {
            return "red";
        }
        else if (version.releaseType === "beta") {
            return "teal";
        }
        else if (version.releaseType === "rc") {
            return "green";
        }
        else if (version.releaseType === "milestone") {
            return "violet";
        }
        return "orange";
    };
    /**
     * Resolves the release version.
     *
     * @return {string} Resolved release version.
     */
    const resolveReleaseVersion = () => {
        var _a, _b, _c;
        if (!version.releaseType && !version.versionNumber) {
            return null;
        }
        let releaseVersion = `${(_a = version.versionNumber) !== null && _a !== void 0 ? _a : ""} ${(_b = version.releaseType) !== null && _b !== void 0 ? _b : ""}`;
        if (version.releaseType === "milestone" && version.milestoneNumber) {
            releaseVersion = `${(_c = version.versionNumber) !== null && _c !== void 0 ? _c : ""} M${version.milestoneNumber}`;
        }
        if (version.textCase === "lowercase") {
            return releaseVersion.toLowerCase();
        }
        else if (version.textCase === "uppercase") {
            return releaseVersion.toUpperCase();
        }
        return releaseVersion.toUpperCase();
    };
    return (React.createElement("div", { className: classNames(className, "product-title"), style: style, "data-testid": testId },
        version && (version.releaseType || version.versionNumber) && (React.createElement("div", { className: "product-title-meta" },
            React.createElement(Label, { color: resolveVersionLabelColor(), className: versionLabelClasses, size: "mini", "data-testid": `${testId}-version` }, resolveReleaseVersion()))),
        React.createElement("div", { className: "product-title-main" },
            logo && logo,
            React.createElement(Heading, { className: classNames(className, "product-title-text"), style: style, "data-testid": `${testId}-title`, compact: true }, name),
            children)));
};
/**
 * Default props for the product brand component.
 */
ProductBrand.defaultProps = {
    "data-testid": "product-brand"
};
//# sourceMappingURL=product-brand.js.map